precision highp float;

// input variables: normal vector and position on surface
varying vec3 v_Normal;
varying vec3 v_Position;

// ambient color
vec3 ca = vec3(0.0, 0.0, 0.6);

// diffuse reflection coefficient
vec3 cd = vec3(0.4, 0.4, 0);

// specular reflection coefficient
vec3 cs = vec3(0.5, 0.0, 0);

void main() {
  // 3d normal vector (perpendicular to surface)
  vec3 n = normalize(v_Normal);

  // 3d point on surface
  vec3 x = v_Position;

  // 3d position of light source
  vec3 p = vec3(0, 0, 0);

  // 3d position of eye (where we are looking from)
  vec3 e = vec3(0, 0, 0);

  // color at the point on the surface
  // (currently set to the ambient color defined above)
  vec3 color = ca;

  // PART 1: add the contribution from diffuse reflection
  vec3 l = normalize(p - x);
  color = color + cd * max(0.0, dot(n, l));

  // PART 2: add the contribution from specular reflection
  vec3 v = normalize(e - x);
  vec3 r = -l + 2.0 * dot(n, l) * n;
  color = color + cs * max(0.0, pow(dot(v, r), 32.0));
  
  // PART 3: "band" the color to make it look like a cartoon
  float n_dot_l = abs(dot(n, l));
  if (n_dot_l < 0.25) {
    n_dot_l = 0.0;
  } else if (n_dot_l < 0.5) {
    n_dot_l = 0.25;
  } else if (n_dot_l < 0.75) {
    n_dot_l = 0.5;
  } else {
    n_dot_l = 0.75;
  }
  // uncomment the next line if you want to use the cartoon version
  //color = ca + cd * n_dot_l;

  // assign the output "fragment" color
  // (this must be set by this program)
  // the last value is the transparency
  gl_FragColor = vec4(color, 1.0);
}

/*
Notes:
- the type of each variable needs to be declared (float, vec3)
- lines must end with a semi-colon
- comments start with //
- I really recommend adding a period (.) after numbers, even for integers.
- useful functions: max(a, b), pow(e, p) (computes e^p)
- conditionals:
     if (condition) {
       // do something if condition is true
     } else if (other_condition) {
       // do something else if other_condition is true
     } else {
      // do something else
     }

Here are some GLSL examples:

vec3 u = vec3(1, 2, 3);
vec3 v = vec3(4, 5, 6);
vec3 u_normalized = normalize(u);
float u_length = length(u);
vec3 u_scaled = 2.0 * u;
vec3 u_plus_v = u + v;
vec3 u_minus_v = u - v;
float u_dot_v = dot(u, v);
vec3 u_cross_v = cross(u, v);
*/